/*
 *
 *  * Copyright (C) 2014 Numatica Corporation.
 *  * All Rights Reserved.
 *  *
 *  * NOTICE:  All information contained herein is, and remains
 *  * the property of Numatica Corporation and its suppliers,
 *  * if any.  The intellectual and technical concepts contained
 *  * herein are proprietary to Numatica Corporation
 *  * and its suppliers and may be covered by U.S. and Foreign Patents,
 *  * patents in process, and are protected by trade secret or copyright law.
 *  * Dissemination of this information or reproduction of this material
 *  * is strictly forbidden unless prior written permission is obtained
 *  * from Numatica Corporation.
 *
 */

package com.numatica.insightmagic;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.CommandLineRunner;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.EnableAutoConfiguration;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.Configuration;
import org.springframework.transaction.annotation.EnableTransactionManagement;

import java.io.IOException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.HashMap;
import java.util.Map;
import java.util.jar.Manifest;
import java.util.stream.Collectors;

@Configuration
@EnableAutoConfiguration
@EnableConfigurationProperties
@EnableTransactionManagement
@ComponentScan
public class Application implements CommandLineRunner {

    private static Logger logger = LoggerFactory.getLogger(Application.class);

    @Value("${app.version:unknown}")
    private String fmVersion;

    @Value("${spring.datasource.url}")
    private String dbUrl;

    public static Map<String, String> manifestEntries = new HashMap<>();

    public static long timeStarted;

    public static void main(final String[] args) {
        SpringApplication springApp = new SpringApplication(Application.class);
        springApp.setShowBanner(false);
        springApp.setLogStartupInfo(false);
        springApp.run(args);
    }

    @Override
    public void run(String... args) {
        readManifest();
        timeStarted = System.currentTimeMillis();
        logger.info("Using db connection: " + dbUrl);
        logger.info("InsightMagic version " + fmVersion + " rev " + manifestEntries.get("Implementation-Build") + " built on " + manifestEntries.get("Implementation-BuildTime") + " started");
    }

    private void readManifest() {
        URLClassLoader cl = (URLClassLoader)getClass().getClassLoader();
        try {
            URL url = cl.findResource("META-INF/MANIFEST.MF");
            Manifest mf = new Manifest(url.openStream());
            manifestEntries = mf.getMainAttributes().entrySet().stream().collect(Collectors.toMap(e -> e.getKey().toString(), e -> e.getValue().toString()));
        } catch (IOException ex) {
            throw new RuntimeException(ex);
        }
    }
}
