/*
 *
 *  * Copyright (C) 2014 Numatica Corporation.
 *  * All Rights Reserved.
 *  *
 *  * NOTICE:  All information contained herein is, and remains
 *  * the property of Numatica Corporation and its suppliers,
 *  * if any.  The intellectual and technical concepts contained
 *  * herein are proprietary to Numatica Corporation
 *  * and its suppliers and may be covered by U.S. and Foreign Patents,
 *  * patents in process, and are protected by trade secret or copyright law.
 *  * Dissemination of this information or reproduction of this material
 *  * is strictly forbidden unless prior written permission is obtained
 *  * from Numatica Corporation.
 *
 */

package com.numatica.insightmagic.common;

import org.apache.commons.lang3.StringUtils;
import org.apache.http.Consts;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.NameValuePair;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.protocol.HTTP;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.slf4j.profiler.Profiler;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.util.Collection;

/**
 * Wrapper for Apache HttpClient.
 *
 * @since 5/22/14
 */
@Component
public class HttpClientHelper {
    private final static String DEFAULT_CONTENT_TYPE = "application/x-www-form-urlencoded";

    private final Logger log = LoggerFactory.getLogger(getClass());

    public String post(final String url, final Collection<NameValuePair> params) {
        CloseableHttpClient httpClient = HttpClientBuilder.create().build();
        Profiler profiler = new Profiler("HTTP POST " + url);

        try {
            HttpPost httpPost = new HttpPost(url);
            httpPost.setHeader(HTTP.CONTENT_TYPE, DEFAULT_CONTENT_TYPE);
            UrlEncodedFormEntity urlEncodedFormEntity = new UrlEncodedFormEntity(params, Consts.UTF_8);
            httpPost.setEntity(urlEncodedFormEntity);

            profiler.start("execute");
            HttpResponse httpResponse = httpClient.execute(httpPost);

            profiler.start("consume");
            HttpEntity responseEntity = httpResponse.getEntity();
            if(null == responseEntity) {
                log.warn("Empty response received for url=[{}]", url);
                return StringUtils.EMPTY;
            }
            String responseBody = EntityUtils.toString(responseEntity);
            EntityUtils.consume(responseEntity);

            profiler.stop().print();
            return responseBody;
        } catch (ClientProtocolException ex) {
            log.error("ClientProtocolException: [{}]", ex);
        } catch (IOException ex) {
            log.error("IOException: [{}]", ex);
        } finally {
            try {
                httpClient.close();
            } catch (IOException ex) {
                log.warn("Failed to close HTTP connection: [{}]", ex);
            }
        }

        return null;
    }
}
