/*
 *
 *  * Copyright (C) 2014 Numatica Corporation.
 *  * All Rights Reserved.
 *  *
 *  * NOTICE:  All information contained herein is, and remains
 *  * the property of Numatica Corporation and its suppliers,
 *  * if any.  The intellectual and technical concepts contained
 *  * herein are proprietary to Numatica Corporation
 *  * and its suppliers and may be covered by U.S. and Foreign Patents,
 *  * patents in process, and are protected by trade secret or copyright law.
 *  * Dissemination of this information or reproduction of this material
 *  * is strictly forbidden unless prior written permission is obtained
 *  * from Numatica Corporation.
 *
 */
package com.numatica.insightmagic.geolocation.service;

import com.hazelcast.core.IMap;
import com.hazelcast.query.SqlPredicate;
import com.numatica.insightmagic.HazelcastConfiguration;
import com.numatica.insightmagic.common.DuplicateBusinessKeyException;
import com.numatica.insightmagic.geolocation.model.GeoLocation;
import com.numatica.insightmagic.service.HazelcastBasedService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.slf4j.profiler.Profiler;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import java.io.*;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Collection;
import java.util.HashMap;
import java.util.Scanner;

/**
 * Created with IntelliJ IDEA.
 * User: Jacek
 * Date: 10/3/14
 * Time: 12:55 PM
 * To change this template use File | Settings | File Templates.
 */
@Service
public class GeoService extends HazelcastBasedService {

    protected final Logger log = LoggerFactory.getLogger(getClass());
    private IMap<Long, GeoLocation> geoMap;

    static Long toNumeric(String ip) {
        Scanner sc = new Scanner(ip).useDelimiter("\\.");
        return
                (sc.nextLong() << 24) +
                        (sc.nextLong() << 16) +
                        (sc.nextLong() << 8) +
                        (sc.nextLong());
    }

    @PostConstruct
    private void init() {

        geoMap = hz.getMap(HazelcastConfiguration.GEO_MAP);
    }

    public GeoLocation getGeoLocationByIpAddress(String ipAddressString) {
        Long ipAddress;
        if (ipAddressString != null) {
            ipAddress = toNumeric(ipAddressString);
        } else {
            return null; //?? not good practice
        }

        Profiler profiler = new Profiler("Get GeoLocation by ipAddress " + ipAddress + " converted from " + ipAddressString);
        Collection<GeoLocation> locations = geoMap.values(new SqlPredicate("ipStartNum < " + ipAddress + " AND ipEndNum > " + ipAddress));

        if (locations.size() > 1) {
            throw new DuplicateBusinessKeyException(locations.toString() + " -> " + ipAddress);
        }
        if (locations.size() > 0) {
            profiler.stop().print();
            return locations.stream().findFirst().get();
        }
        return null;
    }

    public GeoLocation getLocationByStartIp(Long startIpNum) {
        IMap<Long, GeoLocation> geoMap = hz.getMap(HazelcastConfiguration.GEO_MAP);
        return geoMap.get(startIpNum);
    }

    public void loadExternalData(String fileName) {
        Path path = Paths.get(fileName);
        try {
            File f = path.toFile();
            System.out.println(f.getCanonicalPath());
            ingestFile(f, 5);
        } catch (IOException ex) {
            log.error("Error location the file " + ex);
        }
    }

    private void ingestFile(File file, int noOfFields) throws IOException {
        final HashMap<Long, GeoLocation> ipGeoLocationMap = new HashMap<>();
        try (BufferedReader br = new BufferedReader(new InputStreamReader(
                new FileInputStream(file)))) {
            for (String line; (line = br.readLine()) != null; ) {
                final String[] lineContent = line.replaceAll("\"", "").split(",");
                if (lineContent.length == noOfFields) {
                    try {
                        final String ipStart = lineContent[0];
                        final Long ipStartNum = toNumeric(ipStart);
                        final String ipEnd = lineContent[1];
                        final Long ipEndNum = toNumeric(ipEnd);
                        final String countryCode = lineContent[2];
                        final String stateOrProvince = lineContent[3];
                        final String city = lineContent[4];
                        GeoLocation geoLocation = new GeoLocation(ipStartNum, ipStart, ipEndNum, ipEnd, countryCode, stateOrProvince, city);
                        IMap<Long, GeoLocation> geoMap = hz.getMap(HazelcastConfiguration.GEO_MAP);
                        geoMap.putIfAbsent(ipStartNum, geoLocation);
                    } catch (Exception ex) {
                        log.error("Error reading line " + lineContent[0] + lineContent[1], lineContent[2]);
                    }

                }


            }
        }
    }

}
