/*
 *
 *  * Copyright (C) 2014 Numatica Corporation.
 *  * All Rights Reserved.
 *  *
 *  * NOTICE:  All information contained herein is, and remains
 *  * the property of Numatica Corporation and its suppliers,
 *  * if any.  The intellectual and technical concepts contained
 *  * herein are proprietary to Numatica Corporation
 *  * and its suppliers and may be covered by U.S. and Foreign Patents,
 *  * patents in process, and are protected by trade secret or copyright law.
 *  * Dissemination of this information or reproduction of this material
 *  * is strictly forbidden unless prior written permission is obtained
 *  * from Numatica Corporation.
 *
 */
package com.numatica.insightmagic.interaction;

import com.hazelcast.core.IMap;
import com.numatica.insightmagic.HazelcastConfiguration;
import com.numatica.insightmagic.queue.QueueProcessor;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.slf4j.profiler.Profiler;
import org.springframework.beans.factory.annotation.Autowired;

import javax.annotation.PostConstruct;
import java.io.Serializable;


/**
 * Created with IntelliJ IDEA.
 * User: Jacek
 * Date: 10/14/14
 * Time: 4:28 PM
 * To change this template use File | Settings | File Templates.
 */
public class UserActionQueueProcessor extends QueueProcessor {

    protected final Logger log = LoggerFactory.getLogger(getClass());

    protected IMap<Long, UserAction> userActionHistoryMap;

    @Autowired
    private UserActionService userActionService;

    public UserActionQueueProcessor(int threadCount) {
        super(threadCount);
    }

    @PostConstruct
    private void init() {
        start(2, HazelcastConfiguration.USER_ACTION_QUEUE);
        userActionHistoryMap = hz.getMap(HazelcastConfiguration.USER_ACTION_HISTORY_MAP);
    }

    @Override
        public void handleMessage(Serializable message) {
            Profiler profiler = new Profiler("In Handler " + ((UserAction) message).getIpAddress());

            profiler.start("resolveLocation");
            UserAction resolvedUserAction = userActionService.resolveLocation((UserAction) message);
            Long userActionId = System.nanoTime(); //userActionService.generateUserActionId();
            log.info("ResolvedUserAction " + resolvedUserAction.getCountryCode() + " " + resolvedUserAction.getCity() + " " + resolvedUserAction.getStateOrProvince());
            resolvedUserAction.setUaId(userActionId);
            profiler.start("PutIfAbsent");
            userActionHistoryMap.putIfAbsent(userActionId, resolvedUserAction);
            profiler.stop().print();
        }
}